/*
 * Global functions
 * Copyright (C) 2001 Jan Oberlnder <mindriot@gmx.net>
 *
 * This file is part of netload_applet.
 *
 * netload_applet is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: functions.c,v 1.3 2001/10/05 23:43:34 mindriot Exp $
 */

#include <config.h>
#include <unistd.h>
#include <sys/wait.h>
#include "functions.h"

/*
 * Reads a line from a file, automatically sizing the string
 * buffer properly.
 */
gchar *
read_line(FILE *stream)
{
	gchar *line = NULL;
	guint32 len, i;
	guint8 b;

	len = 80;
	line = g_malloc(len + 1);
	i = 0;

	while (!feof(stream)) {
		fread(&b, 1, 1, stream);
		if (b == '\n') {
			line[i] = 0;
			break;
		}
		line[i++] = b;
		if (i == len) {
			len += 80;
			line = g_realloc(line, len + 1);
		}
	}
	line = g_realloc(line, strlen(line) + 1);
	return line;
}

#ifdef BSD
/*
 * Helper function to get the output from netstat into an array of strings
 * each representing a line of output.
 */
gchar **
read_netstat_output(void)
{
	int pd[2], pid;
	char *buf;
	int buf_size, buf_len, i;

#if (DEBUG == 1)
	printf("Reading output for device %s.\n", display_scheme.device);
#endif

	pipe((int *) &pd);
	if (!(pid = fork())) {
		close(pd[0]);  /* Reading end closed         */
		dup2(pd[1],1); /* Bind writing end to stdout */
		/* run netstat */
		execlp("netstat", "netstat", "-I", display_scheme.device, "-b", NULL);
	}
	close(pd[1]);          /* Writing end closed         */
	buf_size = 128;
	buf_len = 0;
	buf = (char *) malloc(buf_size+1); /* 1 extra byte for \0     */
	
	while ((i = read(pd[0], &buf[buf_size-128], 128)) > 0) {
#if (DEBUG == 1)
		printf("%i,",i);
#endif
		buf_len += i;
		if (buf_len > buf_size-128) {
			buf_size += 128;
			buf = (char *)realloc(buf, buf_size+1);
		}
	}
#if (DEBUG == 1)
	printf("%i.\n",i);
#endif
	buf[buf_len] = 0;      /* ensure ASCIZ               */
	close (pd[0]);         /* close                      */
	wait(NULL);            /* Wait for child to finish   */
	/*
	 * Split into separate lines
	 */
	return g_strsplit(buf, "\n", 0);
}
#endif

/*
 * Gets the current byte counts.
 */
void
get_byte_counts(void)
{
	gchar **split_line;
	gint32 rx_byte_count = 0, tx_byte_count = 0;
#if defined (BSD)
	gint i;
	guint32 rxtemp, txtemp;
#else
	FILE *stream;
	gchar *line;
#endif

#if defined (BSD)
	DEBUG_MESSAGE ("get_byte_counts: reading from netstat output.\n");
	split_line = read_netstat_output();
	if (!split_line) {
		iface_down = TRUE;
		rx_byte_count = 0;
		tx_byte_count = 0;
	} else {
		rx_byte_count = 0;
		tx_byte_count = 0;
		i = 0;
		while (split_line[i]) {
			if (i == 0) {
				i++;
				continue;
			}
			rxtemp = txtemp = 0;
			sscanf(split_line[i++], "%*s %*s %*s %*s %*s %*s %i %*s %*s %i", &rxtemp, &txtemp);
			rx_byte_count += rxtemp;
			tx_byte_count += txtemp;
		}
		g_free(split_line);
	}
#else /* LINUX variant */
	DEBUG_MESSAGE ("get_byte_counts: reading from /proc/net/dev.\n");
	line = NULL;
	iface_down = FALSE;
	if ((stream = fopen("/proc/net/dev", "r")) == NULL) {
		DEBUG_MESSAGE ("ERROR: could not open /proc/net/dev. Sorry.\n");
		iface_down = TRUE;
		rx_byte_count = 0;
		tx_byte_count = 0;
	} else {
		do {
			if (line) g_free (line);
			line = read_line (stream);
			if (feof (stream)) {
				iface_down = TRUE;
				break;
			}
		} while (strstr (line, display_scheme.device) == NULL);
	}

	if (!iface_down) {
		split_line = g_strsplit (line, ":", 1);
		sscanf (split_line[1], "%i %*i %*i %*i %*i %*i %*i %*i %i",
			&rx_byte_count, &tx_byte_count);
#if (DEBUG == 1)
		printf("get_byte_counts: read rx=%i, tx=%i.\n", rx_byte_count, tx_byte_count);
#endif
		g_strfreev (split_line);
	} else {
		/*
		 * Set to zero if interface down or non-existent.
		 */
		rx_byte_count = 0;
		tx_byte_count = 0;
	}

	g_free (line);
	if (stream) {
		fclose (stream);
	}
#endif
	rx_delta_counts[display_scheme.width-3] = rx_byte_count - rx_byte_counts[display_scheme.width-3];
	tx_delta_counts[display_scheme.width-3] = tx_byte_count - tx_byte_counts[display_scheme.width-3];
	rx_byte_counts[display_scheme.width-3] = rx_byte_count;
	tx_byte_counts[display_scheme.width-3] = tx_byte_count;

#if (DEBUG == 1)
	if (iface_down) {
		printf ("get_byte_counts: interface down.\n");
	} else {
		printf ("get_byte_counts: rx_delta=%i, tx_delta=%i.\n",
			rx_delta_counts[display_scheme.width-3],
			tx_delta_counts[display_scheme.width-3]);
	}
#endif
}

/*
 * Initializes byte count fields so that the first delta
 * is correct (when we jump from 0 bytes to the most likely already large amount
 * present in the interface stats).
 * This function is called from resize_count_fields if no previous applet
 * width is present, which means it has not been initialized before. When
 * the monitored device is changed, interface.c's update_scheme sets old_width back
 * to 0 to make sure init_byte_counts is called again. Kind of a hack... :)
 */
void
init_byte_counts(void)
{
	gchar **split_line;
#if defined (BSD)
	gint i;
	guint32 rxtemp, txtemp;
#else
	FILE *stream;
	gchar *line;
#endif

	iface_down = FALSE;

#if defined (BSD)
	DEBUG_MESSAGE ("init_byte_counts: reading from netstat output.\n");
	rx_byte_counts[display_scheme.width-3] = 0;
	tx_byte_counts[display_scheme.width-3] = 0;
	split_line = read_netstat_output();
	if (!split_line) {
		iface_down = TRUE;
	} else {
		i = 0;
		while (split_line[i]) {
			if (i == 0) {
				i++;
				continue;
			}
			rxtemp = txtemp = 0;
			sscanf(split_line[i++], "%*s %*s %*s %*s %*s %*s %i %*s %*s %i", &rxtemp, &txtemp);
			rx_byte_counts[display_scheme.width-3] += rxtemp;
			tx_byte_counts[display_scheme.width-3] += txtemp;
		}
		g_free(split_line);
	}
#else /* LINUX variant */
	DEBUG_MESSAGE ("init_byte_counts: reading from /proc/net/dev.\n");
	line = NULL;
	if ((stream = fopen("/proc/net/dev", "r")) == NULL) {
		iface_down = TRUE;
	} else {
		do {
			if (line) g_free (line);
			line = read_line (stream);
			if (feof (stream)) {
				iface_down = TRUE;
				break;
			}
		} while (strstr (line, display_scheme.device) == NULL);
	}

	if (!iface_down) {
		split_line = g_strsplit(line, ":", 1);
		sscanf (split_line[1], "%i %*i %*i %*i %*i %*i %*i %*i %i",
			&rx_byte_counts[display_scheme.width-3],
			&tx_byte_counts[display_scheme.width-3]);
		g_strfreev (split_line);
	} else {
		/*
		 * Set to zero if interface down or non-existent.
		 */
		rx_byte_counts[display_scheme.width-3] = 0;
		tx_byte_counts[display_scheme.width-3] = 0;
	}

	g_free (line);
	if (stream) {
		fclose (stream);
	}
#endif

#if (DEBUG == 1)
	if (iface_down) {
		printf ("init_byte_counts: interface down.\n");
	} else {
		printf ("init_byte_counts: Receive count: %i, Transmit count: %i.\n",
			rx_byte_counts[display_scheme.width-3],
			tx_byte_counts[display_scheme.width-3]);
	}
#endif
}

/*
 * reads the available network devices.
 */
gchar **
get_available_devices (void)
{
	gchar **device_names=NULL;
	gchar *line;
#if defined (LINUX)
	FILE *stream;
	gchar **split_line;
	guint list_size, list_pos;

	DEBUG_MESSAGE("get_available_devices: (Linux) Reading from /proc/net/dev.\n");

	list_size = 4;
	device_names = g_malloc0 ((list_size+1) * sizeof(gchar *));
	list_pos = 0;

	if ((stream = fopen("/proc/net/dev", "r")) == NULL) {
	        DEBUG_MESSAGE("ERROR: could not open /proc/net/dev. Sorry.\n");
	} else {
		do {
			line = read_line (stream);
			if (strchr (line, ':')) {
				split_line = g_strsplit(line, ":", 1);
				while (split_line[0][0] == ' ')
					split_line[0] = g_strchug (split_line[0]);
#if (DEBUG == 1)
				printf ("get_available_devices: read '%s'.\n", split_line[0]);
#endif
				device_names[list_pos++] = g_strdup (split_line[0]);
				if (list_pos == list_size) {
					list_size += 4;
					device_names = g_realloc (device_names, (list_size+1) * sizeof(gchar *));
				}
				g_strfreev (split_line);
			}
			g_free (line);
		} while (!feof(stream));
	}
	device_names[list_pos] = NULL;
#elif defined (BSD)
	int pd[2], pid;
	char *buf;
	int buf_size, buf_len, i;

	DEBUG_MESSAGE("get_available_devices: (BSD) Reading from ifconfig -l.\n");
	pipe((int *) &pd);
	if (!(pid = fork())) {
		close(pd[0]);  /* Reading end closed         */
		dup2(pd[1],1); /* Bind writing end to stdout */
		/* run ifconfig */
		execlp("ifconfig", "ifconfig", "-l", NULL);
	}
	close(pd[1]);          /* Writing end closed         */
	buf_size = 128;
	buf_len = 0;
	buf = (char *) malloc(buf_size+1); /* 1 extra byte for \0     */
	while ((i = read(pd[0], &buf[buf_size-128], 128)) > 0) {
		buf_size += 128;
		buf_len += i;
		buf = (char *)realloc(buf, buf_size+1);
	}
	buf[buf_len] = 0;      /* ensure ASCIZ               */
	buf = g_strstrip(buf); /* Strip whitespace           */
	device_names = g_strsplit(buf, " ", 0);
	close (pd[0]);         /* close                      */
	wait(NULL);            /* Wait for child to finish   */
#else
	device_names = g_malloc0(sizeof(gchar *));
#endif

#if (DEBUG == 1)
	line = g_strjoinv (",", device_names);
	printf ("The devices are: '%s'.\n", line);
	g_free (line);
#endif
	return device_names;
}

void
shift_values(void)
{
	guint32 i;

	DEBUG_MESSAGE ("shift_values: shifting values.\n");
	for (i = 0; i < display_scheme.width-3; i++) {
		rx_byte_counts[i] = rx_byte_counts[i+1];
		tx_byte_counts[i] = tx_byte_counts[i+1];
		rx_delta_counts[i] = rx_delta_counts[i+1];
		tx_delta_counts[i] = tx_delta_counts[i+1];
		rx_values[i] = rx_values[i+1];
		tx_values[i] = tx_values[i+1];
	}
}

void
scale_values(void)
{
	gint i;
	guint32 rx_avg, tx_avg;

	largest = rx_avg = tx_avg = 0;
	for (i = 0; i < display_scheme.width-2; i++) {
		if (rx_delta_counts[i] > largest) largest = rx_delta_counts[i];
		rx_avg += rx_delta_counts[i];
		if (tx_delta_counts[i] > largest) largest = tx_delta_counts[i];
		tx_avg += tx_delta_counts[i];
	}

	for (i = 0; i < display_scheme.width-2; i++) {
		rx_values[i] = (rx_delta_counts[i] * (display_scheme.height-2)) / (largest + 1);
		tx_values[i] = (tx_delta_counts[i] * (display_scheme.height-2)) / (largest + 1);
	}

	rx_average = ((rx_avg / (display_scheme.width-2)) * (display_scheme.height-2)) / (largest + 1);
	tx_average = ((tx_avg / (display_scheme.width-2)) * (display_scheme.height-2)) / (largest + 1);

#if (DEBUG == 1)
	printf("scale_values: largest=%i, rx_avg=%i, tx_avg=%i.\n", largest, rx_average, tx_average);
#endif
}

/*
 * updates count fields to new size.
 */
void
resize_count_fields (void)
{
	gint32 i, j;
	guint32 *rx_byte_counts_new,
		*tx_byte_counts_new,
		*rx_delta_counts_new,
		*tx_delta_counts_new,
		*rx_values_new,
		*tx_values_new;

	DEBUG_MESSAGE ("resize_count_fields: re-initializing all fields.\n");

	g_assert(display_scheme.width - 2 > 0);

	/*
	 * Allocate new arrays
	 */
	rx_byte_counts_new = g_malloc (sizeof(guint32) * (display_scheme.width - 2));
	tx_byte_counts_new = g_malloc (sizeof(guint32) * (display_scheme.width - 2));
	rx_delta_counts_new = g_malloc (sizeof(guint32) * (display_scheme.width - 2));
	tx_delta_counts_new = g_malloc (sizeof(guint32) * (display_scheme.width - 2));
	rx_values_new = g_malloc (sizeof(guint32) * (display_scheme.width - 2));
	tx_values_new = g_malloc (sizeof(guint32) * (display_scheme.width - 2));

	/*
	 * Transfer old values
	 */
	for (i=0; i<display_scheme.width-2; i++) {
		rx_byte_counts_new[i] = tx_byte_counts_new[i]
			= rx_delta_counts_new[i] = tx_delta_counts_new[i]
			= rx_values_new[i] = tx_values_new[i] = 0;
	}
	if (oldwidth != 0) {
		j = oldwidth-display_scheme.width;
		for (i=0; i<display_scheme.width-2; i++) {
			if (i+j >= 0) {
				if (rx_byte_counts) rx_byte_counts_new[i] = rx_byte_counts[i+j];
				if (tx_byte_counts) tx_byte_counts_new[i] = tx_byte_counts[i+j];
				if (rx_delta_counts) rx_delta_counts_new[i] = rx_delta_counts[i+j];
				if (tx_delta_counts) tx_delta_counts_new[i] = tx_delta_counts[i+j];
				if (rx_values) rx_values_new[i] = rx_values[i+j];
				if (tx_values) tx_values_new[i] = tx_values[i+j];
			} else {
				if (rx_byte_counts) rx_byte_counts_new[i] = rx_byte_counts[0];
				if (tx_byte_counts) tx_byte_counts_new[i] = tx_byte_counts[0];
			}
		}
	} else {
		/*
		 * no previous counts, so do an Init. We need to point the arrays to our new arrays,
		 * and set them to NULL again so they don't get freed... sort of ugly hack...
		 */
		rx_byte_counts = rx_byte_counts_new;
		tx_byte_counts = tx_byte_counts_new;
		init_byte_counts();
		rx_byte_counts = NULL;
		tx_byte_counts = NULL;
	}
	oldwidth = display_scheme.width;

	/*
	 * Done, clear old ones
	 */
	if (rx_byte_counts) g_free (rx_byte_counts);
	if (tx_byte_counts) g_free (tx_byte_counts);
	if (rx_delta_counts) g_free (rx_delta_counts);
	if (tx_delta_counts) g_free (tx_delta_counts);
	if (rx_values) g_free (rx_values);
	if (tx_values) g_free (tx_values);
	rx_byte_counts = rx_byte_counts_new;
	tx_byte_counts = tx_byte_counts_new;
	rx_delta_counts = rx_delta_counts_new;
	tx_delta_counts = tx_delta_counts_new;
	rx_values = rx_values_new;
	tx_values = tx_values_new;

	/*
	 * scale correctly
	 */
	scale_values();
}

/*
 * This sets default values for the display scheme.
 * NOTE: this does NOT initialize the font/color fields!
 */
void
set_default_display_scheme (void)
{
	display_scheme = default_display_scheme;

	display_scheme.rx_load_font_name = g_strdup_printf ("-Adobe-Helvetica-Medium-R-Normal--8-80-75-75-P-46-ISO8859-1");
	display_scheme.tx_load_font_name = g_strdup_printf ("-Adobe-Helvetica-Medium-R-Normal--8-80-75-75-P-46-ISO8859-1");
	display_scheme.max_load_font_name = g_strdup_printf ("-Adobe-Helvetica-Medium-R-Normal--8-80-75-75-P-46-ISO8859-1");
	display_scheme.dev_font_name = g_strdup_printf ("-Adobe-Helvetica-Medium-R-Normal--8-80-75-75-P-46-ISO8859-1");
	display_scheme.device = g_strdup_printf ("eth0");
}

/*
 * This frees all data associated with the display scheme.
 * It unrefs and frees the fonts and font names.
 */
void
free_display_scheme_data (void)
{
	/*
	 * unref fonts
	 */
	gdk_font_unref (display_scheme.rx_load_font);
	gdk_font_unref (display_scheme.tx_load_font);
	gdk_font_unref (display_scheme.max_load_font);
	gdk_font_unref (display_scheme.dev_font);

	/*
	 * free names
	 */
	g_free (display_scheme.rx_load_font_name);
	g_free (display_scheme.tx_load_font_name);
	g_free (display_scheme.max_load_font_name);
	g_free (display_scheme.dev_font_name);
	g_free (display_scheme.device);
}

/*
 * initializes the display scheme, i.e. it makes sure that all values are valid
 * and it loads the fonts and the colors.
 */
void
init_display_scheme (void)
{
	GdkColormap *cmap;

	/*
	 * load and ref all the fonts.
	 */
	DEBUG_MESSAGE ("init_display_scheme: Initializing fonts.\n");
	if (!(display_scheme.rx_load_font = gdk_font_load (display_scheme.rx_load_font_name)))
		g_error("Can't load font for receive load display!\n");
	if (!(display_scheme.tx_load_font = gdk_font_load (display_scheme.tx_load_font_name)))
		g_error("Can't load font for transmit load display!\n");
	if (!(display_scheme.max_load_font = gdk_font_load (display_scheme.max_load_font_name)))
		g_error("Can't load font fot maximum load display!\n");
	if (!(display_scheme.dev_font = gdk_font_load (display_scheme.dev_font_name)))
		g_error("Can't load font for device name display!\n");
	gdk_font_ref (display_scheme.rx_load_font);
	gdk_font_ref (display_scheme.tx_load_font);
	gdk_font_ref (display_scheme.max_load_font);
	gdk_font_ref (display_scheme.dev_font);

	/*
	 * if we have Gnome 1.2 panel size features, use them to get standard width+height.
	 * otherwise, use 48x48 if no fixed size is given.
	 */
#ifndef HAVE_PANEL_PIXEL_SIZE
	DEBUG_MESSAGE ("init_display_scheme: Gnome 1.2 panel size feature unavailable, setting to fixed width.\n");
	display_scheme.fixed_width = TRUE;
	display_scheme.fixed_height = TRUE;
#else
	DEBUG_MESSAGE ("init_display_scheme: Gnome 1.2 panel size feature available.\n");
#endif
	/*
	 * always set reasonable default width/height and timeout.
	 */
	if (display_scheme.width == 0)
		display_scheme.width = 48;
	if (display_scheme.height == 0)
		display_scheme.height = 48;
	if (display_scheme.timeout < 200)
		display_scheme.timeout = 200;

	/*
	 * allocate byte count fields once
	 */
	resize_count_fields ();

	/*
	 * allocate all needed colors.
	 */
	DEBUG_MESSAGE ("init_display_scheme: Allocating all colors.\n");
	cmap = gdk_colormap_new (gdk_visual_get_best(), TRUE);
	gdk_colormap_alloc_color (cmap, &display_scheme.background_color, FALSE, TRUE);
	gdk_colormap_alloc_color (cmap, &display_scheme.rx_graph_color, FALSE, TRUE);
	gdk_colormap_alloc_color (cmap, &display_scheme.tx_graph_color, FALSE, TRUE);
	gdk_colormap_alloc_color (cmap, &display_scheme.rx_avg_color, FALSE, TRUE);
	gdk_colormap_alloc_color (cmap, &display_scheme.tx_avg_color, FALSE, TRUE);
	gdk_colormap_alloc_color (cmap, &display_scheme.rx_load_color, FALSE, TRUE);
	gdk_colormap_alloc_color (cmap, &display_scheme.tx_load_color, FALSE, TRUE);
	gdk_colormap_alloc_color (cmap, &display_scheme.max_load_color, FALSE, TRUE);
	gdk_colormap_alloc_color (cmap, &display_scheme.dev_color, FALSE, TRUE);
	gdk_colormap_alloc_color (cmap, &display_scheme.top_edge_color, FALSE, TRUE);
	gdk_colormap_alloc_color (cmap, &display_scheme.bottom_edge_color, FALSE, TRUE);
}

gint
gtk_option_menu_get_history (GtkOptionMenu *option_menu)
{
	GtkWidget *active_widget;
        
	g_return_val_if_fail (GTK_IS_OPTION_MENU (option_menu), -1);
        
	active_widget = gtk_menu_get_active (GTK_MENU (option_menu->menu));

	if (active_widget)
		return g_list_index (GTK_MENU_SHELL (option_menu->menu)->children,
				     active_widget);
	else
		return -1;
}

gboolean
get_config (void)
{
#if (DEBUG == 1)
	printf ("get_config: reading user configuration from %s.\n", APPLET_WIDGET (applet)->privcfgpath);
#endif

	gnome_config_push_prefix (APPLET_WIDGET (applet)->privcfgpath);

	if (!gnome_config_has_section ("main")) {
		DEBUG_MESSAGE ("get_config: No configuration present.\n");

		gnome_config_pop_prefix ();
		return FALSE;
	}

	/*
	 * OK, now we can read our configuration.
	 */

	display_scheme.rx_load_font_name = gnome_config_get_string ("main/rx_load_font_name");
	display_scheme.tx_load_font_name = gnome_config_get_string ("main/tx_load_font_name");
	display_scheme.max_load_font_name = gnome_config_get_string ("main/max_load_font_name");
	display_scheme.dev_font_name = gnome_config_get_string ("main/dev_font_name");

	display_scheme.background_color.red = gnome_config_get_int ("main/background_color_red");
	display_scheme.background_color.green = gnome_config_get_int ("main/background_color_green");
	display_scheme.background_color.blue = gnome_config_get_int ("main/background_color_blue");
	display_scheme.rx_graph_color.red = gnome_config_get_int ("main/rx_graph_color_red");
	display_scheme.rx_graph_color.green = gnome_config_get_int ("main/rx_graph_color_green");
	display_scheme.rx_graph_color.blue = gnome_config_get_int ("main/rx_graph_color_blue");
	display_scheme.tx_graph_color.red = gnome_config_get_int ("main/tx_graph_color_red");
	display_scheme.tx_graph_color.green = gnome_config_get_int ("main/tx_graph_color_green");
	display_scheme.tx_graph_color.blue = gnome_config_get_int ("main/tx_graph_color_blue");
	display_scheme.rx_avg_color.red = gnome_config_get_int ("main/rx_avg_color_red");
	display_scheme.rx_avg_color.green = gnome_config_get_int ("main/rx_avg_color_green");
	display_scheme.rx_avg_color.blue = gnome_config_get_int ("main/rx_avg_color_blue");
	display_scheme.tx_avg_color.red = gnome_config_get_int ("main/tx_avg_color_red");
	display_scheme.tx_avg_color.green = gnome_config_get_int ("main/tx_avg_color_green");
	display_scheme.tx_avg_color.blue = gnome_config_get_int ("main/tx_avg_color_blue");
	display_scheme.rx_load_color.red = gnome_config_get_int ("main/rx_load_color_red");
	display_scheme.rx_load_color.green = gnome_config_get_int ("main/rx_load_color_green");
	display_scheme.rx_load_color.blue = gnome_config_get_int ("main/rx_load_color_blue");
	display_scheme.tx_load_color.red = gnome_config_get_int ("main/tx_load_color_red");
	display_scheme.tx_load_color.green = gnome_config_get_int ("main/tx_load_color_green");
	display_scheme.tx_load_color.blue = gnome_config_get_int ("main/tx_load_color_blue");
	display_scheme.max_load_color.red = gnome_config_get_int ("main/max_load_color_red");
	display_scheme.max_load_color.green = gnome_config_get_int ("main/max_load_color_green");
	display_scheme.max_load_color.blue = gnome_config_get_int ("main/max_load_color_blue");
	display_scheme.dev_color.red = gnome_config_get_int ("main/dev_color_red");
	display_scheme.dev_color.green = gnome_config_get_int ("main/dev_color_green");
	display_scheme.dev_color.blue = gnome_config_get_int ("main/dev_color_blue");
	display_scheme.top_edge_color.red = gnome_config_get_int ("main/top_edge_color_red");
	display_scheme.top_edge_color.green = gnome_config_get_int ("main/top_edge_color_green");
	display_scheme.top_edge_color.blue = gnome_config_get_int ("main/top_edge_color_blue");
	display_scheme.bottom_edge_color.red = gnome_config_get_int ("main/bottom_edge_color_red");
	display_scheme.bottom_edge_color.green = gnome_config_get_int ("main/bottom_edge_color_green");
	display_scheme.bottom_edge_color.blue = gnome_config_get_int ("main/bottom_edge_color_blue");

	display_scheme.rx_load_position = gnome_config_get_int ("main/rx_load_position");
	display_scheme.tx_load_position = gnome_config_get_int ("main/tx_load_position");
	display_scheme.max_load_position = gnome_config_get_int ("main/max_load_position");
	display_scheme.dev_position = gnome_config_get_int ("main/dev_position");

	display_scheme.show_rx_graph = gnome_config_get_bool ("main/show_rx_graph");
	display_scheme.show_tx_graph = gnome_config_get_bool ("main/show_tx_graph");
	display_scheme.show_rx_avg = gnome_config_get_bool ("main/show_rx_avg");
	display_scheme.show_tx_avg = gnome_config_get_bool ("main/show_tx_avg");
	display_scheme.show_rx_load = gnome_config_get_bool ("main/show_rx_load");
	display_scheme.show_tx_load = gnome_config_get_bool ("main/show_tx_load");
	display_scheme.show_max_load = gnome_config_get_bool ("main/show_max_load");
	display_scheme.show_dev = gnome_config_get_bool ("main/show_dev");

	display_scheme.rx_graph_style = gnome_config_get_int ("main/rx_graph_style");
	display_scheme.tx_graph_style = gnome_config_get_int ("main/tx_graph_style");
	display_scheme.graph_order = gnome_config_get_int ("main/graph_order");

	display_scheme.width = gnome_config_get_int ("main/width");
	display_scheme.height = gnome_config_get_int ("main/height");
	display_scheme.fixed_width = gnome_config_get_bool ("main/fixed_width");
	display_scheme.fixed_height = gnome_config_get_bool ("main/fixed_height");

	display_scheme.device = gnome_config_get_string ("main/device");
	display_scheme.timeout = gnome_config_get_int ("main/timeout");

	gnome_config_pop_prefix ();

	return TRUE;
}
